@@ -21,12 +21,17 @@ from uuid_extensions import uuid7
 from timbal.types.events.chunk import ChunkEvent
 
 from ...errors import AgentError, EarlyExit
-from ...state import set_run_context
+from ...state import get_run_context, set_run_context
 from ...state.context import RunContext
 from ...state.savers.base import BaseSaver
 from ...state.snapshot import Snapshot
 from ...steps.llms.router import llm_router
-from ...types.chat.content import FileContent, ToolResultContent, ToolUseContent
+from ...types.chat.content import (
+    FileContent, 
+    TextContent,
+    ToolResultContent, 
+    ToolUseContent
+)
 from ...types.events import Event, OutputEvent, StartEvent
 from ...types.file import File
 from ...types.llms.usage import acc_usage
@@ -154,9 +159,6 @@ class Agent(BaseStep):
             return
 
         self.state_saver = state_saver
-        if self.state_saver is not None:
-            self._is_state_saver_get_async = inspect.iscoroutinefunction(self.state_saver.get_last)
-            self._is_state_saver_put_async = inspect.iscoroutinefunction(self.state_saver.put)
 
         self._load_tools(tools)
         self.max_iter = max_iter
@@ -335,18 +337,18 @@ class Agent(BaseStep):
 
         system_prompt = kwargs.pop("system_prompt", None)
 
-        context_files = []
-        for message in messages:
-            for content in message.content:
-                if isinstance(content, FileContent):
-                    file = File.serialize(content.file)
-                    context_files.append(file)
+        # context_files = []
+        # for message in messages:
+        #     for content in message.content:
+        #         if isinstance(content, FileContent):
+        #             file = File.serialize(content.file)
+        #             context_files.append(file)
 
-        if len(context_files):
-            if system_prompt is None:
-                system_prompt = f"<context_data>{''.join(f'\n- {f}' for f in context_files)}\n</context_data>"
-            else:
-                system_prompt += f"\n\n<context_data>{''.join(f'\n- {f}' for f in context_files)}\n</context_data>"
+        # if len(context_files):
+        #     if system_prompt is None:
+        #         system_prompt = f"<context_data>{''.join(f'\n- {f}' for f in context_files)}\n</context_data>"
+        #     else:
+        #         system_prompt += f"\n\n<context_data>{''.join(f'\n- {f}' for f in context_files)}\n</context_data>"
 
         llm_error = None
         llm_message = None
@@ -494,9 +496,13 @@ class Agent(BaseStep):
 
             # Handle the case where the tool already returns an LLM message. We need to modify it so it represents the result of a tool call.
             if not isinstance(tool_output, Message):
+                if isinstance(tool_output, File):
+                    tool_output_content = FileContent(file=tool_output)
+                else:
+                    tool_output_content = TextContent(text=str(tool_output))
                 tool_output = Message.validate({
                     "role": "user",
-                    "content": str(tool_output),
+                    "content": tool_output_content,
                 })
             
         except Exception as err:
@@ -527,16 +533,14 @@ class Agent(BaseStep):
         )
 
 
-    async def _run_local(
-        self, 
-        context: RunContext | None = None,
-        **kwargs: Any,
-    ) -> Any:
+    async def _run_local(self, **kwargs: Any) -> Any:
         """Runs the agent locally."""
         t0 = int(time.time() * 1000)
 
+        context = get_run_context()
+
         # Copy the input as is, so we save the traces without validated data and defaults.
-        agent_input = dump(kwargs, context=context)
+        agent_input = await dump(kwargs)
 
         # Load the memory.
         # Always do this first to ensure even if validation fails we can carry the memory to the next run.
@@ -545,10 +549,7 @@ class Agent(BaseStep):
         context.data["memory"] = messages
         if self.state_saver is not None and context.parent_id is not None:
             try:
-                if self._is_state_saver_get_async:
-                    last_snapshot = await self.state_saver.get_last(path=self.path, context=context)
-                else:
-                    last_snapshot = self.state_saver.get_last(path=self.path, context=context)
+                last_snapshot = await self.state_saver.get_last(path=self.path)
             except Exception as err:
                 logger.error("get_memory_error", err=err)
                 last_snapshot = None
@@ -567,14 +568,13 @@ class Agent(BaseStep):
         for k, v in kwargs.items():
             context.data[k] = v
 
-        agent_start_event = StartEvent(
+        agent_start_event = await StartEvent.build(
             run_id=context.id,
             path=self.path,
             status_text="Starting...",
         )
-        agent_start_event_dump = dump(agent_start_event, context=context)
 
-        logger.info("start_event", start_event=agent_start_event_dump)
+        logger.info("start_event", start_event=agent_start_event.dump)
         yield agent_start_event
 
         # Aggregated traces and usage for the entire run.
@@ -592,7 +592,7 @@ class Agent(BaseStep):
             if "prompt" in kwargs:
                 # We pre-validate the prompt field as a message. Frontend expects this to be a Message instance.
                 kwargs["prompt"] = Message.validate(kwargs["prompt"])
-                agent_input = dump(kwargs, context=context)
+                agent_input = await dump(kwargs)
                 messages.append(kwargs.pop("prompt"))
 
             if not len(messages):
@@ -625,10 +625,7 @@ class Agent(BaseStep):
                 
                 # We don't want to cancel the execution if this errors. 
                 try:
-                    if self._is_state_saver_put_async:
-                        await self.state_saver.put(snapshot=snapshot, context=context)
-                    else:
-                        self.state_saver.put(snapshot=snapshot, context=context)
+                    await self.state_saver.put(snapshot=snapshot)
                 except Exception as err:
                     logger.error("put_memory_error", err=err)
             return
@@ -657,10 +654,7 @@ class Agent(BaseStep):
                 
                 # We don't want to cancel the execution if this errors. 
                 try:
-                    if self._is_state_saver_put_async:
-                        await self.state_saver.put(snapshot=snapshot, context=context)
-                    else:
-                        self.state_saver.put(snapshot=snapshot, context=context)
+                    await self.state_saver.put(snapshot=snapshot)
                 except Exception as err:
                     logger.error("put_memory_error", err=err)
 
@@ -668,14 +662,13 @@ class Agent(BaseStep):
 
         # Run an llm first (that is, the handler_fn for the llm gateway step).
         llm_i_path = f"{self.path}.llm-0"
-        llm_start_event = StartEvent(
+        llm_start_event = await StartEvent.build(
             run_id=context.id,
             path=llm_i_path,
             status_text="Thinking...",
         )
-        llm_start_event_dump = dump(llm_start_event, context=context)
 
-        logger.info("start_event", start_event=llm_start_event_dump)
+        logger.info("start_event", start_event=llm_start_event.dump)
         yield llm_start_event
 
         async for llm_output in self._run_llm(
@@ -688,28 +681,16 @@ class Agent(BaseStep):
             else:
                 llm_chunk = llm_output
 
-                llm_chunk_event = ChunkEvent(
+                llm_chunk_event = await ChunkEvent.build(
                     run_id=context.id,
                     path=llm_i_path,
                     chunk=llm_chunk.output,
                 )
-                llm_chunk_event_dump = dump(llm_chunk_event, context=context)
 
-                logger.info("chunk_event", chunk_event=llm_chunk_event_dump)
+                logger.info("chunk_event", chunk_event=llm_chunk_event.dump)
                 yield llm_chunk_event
 
-                # # If the LLM is returning a stream, that indicates it's not going to use a tool.
-                # # We're safe streaming the chunks as the final agent response.
-                # agent_chunk_event = ChunkEvent(
-                #     run_id=context.id,
-                #     path=self.path,
-                #     chunk=llm_chunk.output,
-                # )
-
-                # logger.info("chunk_event", chunk_event=agent_chunk_event)
-                # yield agent_chunk_event
-
-        llm_output_event = OutputEvent(
+        llm_output_event = await OutputEvent.build(
             run_id=context.id,
             path=llm_i_path,
             input=llm_result.input,
@@ -719,13 +700,12 @@ class Agent(BaseStep):
             t1=llm_result.t1,
             usage=llm_result.usage,
         )
-        llm_output_event_dump = dump(llm_output_event, context=context)
 
-        logger.info("output_event", output_event=llm_output_event_dump)
+        logger.info("output_event", output_event=llm_output_event.dump)
         yield llm_output_event
 
         # Store the trace of the LLM step.
-        run_steps[llm_i_path] = llm_output_event_dump
+        run_steps[llm_i_path] = llm_output_event.dump
 
         # Aggregate the usage of the LLM step.
         for k, v in llm_result.usage.items():
@@ -754,10 +734,7 @@ class Agent(BaseStep):
                 
                 # We don't want to cancel the execution if this errors. 
                 try:
-                    if self._is_state_saver_put_async:
-                        await self.state_saver.put(snapshot=snapshot, context=context)
-                    else:
-                        self.state_saver.put(snapshot=snapshot, context=context)
+                    await self.state_saver.put(snapshot=snapshot)
                 except Exception as err:
                     logger.error("put_memory_error", err=err)
 
@@ -785,15 +762,14 @@ class Agent(BaseStep):
                 assert len(self.tools) > tool_idx, f"Tool {tool_call.name} not found at index {tool_idx}."
                 tool = self.tools[tool_idx]
 
-                tool_start_event = StartEvent(
+                tool_start_event = await StartEvent.build(
                     run_id=context.id,
                     path=f"{tool.path}-{tool_call.id}",
                     # TODO Review this one.
                     status_text=f"Running tool: {tool.path}...",
                 )
-                tool_start_event_dump = dump(tool_start_event, context=context)
 
-                logger.info("start_event", start_event=tool_start_event_dump)
+                logger.info("start_event", start_event=tool_start_event.dump)
                 yield tool_start_event
 
                 tool_task = asyncio.create_task(
@@ -821,7 +797,7 @@ class Agent(BaseStep):
                 })
                 messages.append(tool_result_message)
                 
-                tool_output_event = OutputEvent(
+                tool_output_event = await OutputEvent.build(
                     run_id=context.id,
                     path=f"{tool.path}-{tool_call.id}",
                     input=tool_result.input,
@@ -831,13 +807,12 @@ class Agent(BaseStep):
                     t1=tool_result.t1,
                     usage=tool_result.usage,
                 )
-                tool_output_event_dump = dump(tool_output_event, context=context)
 
-                logger.info("output_event", output_event=tool_output_event_dump)
+                logger.info("output_event", output_event=tool_output_event.dump)
                 yield tool_output_event
 
                 # Store the trace of the LLM step.
-                run_steps[f"{tool.path}-{tool_call.id}"] = tool_output_event_dump
+                run_steps[f"{tool.path}-{tool_call.id}"] = tool_output_event.dump
 
                 # Aggregate the usage of the tool step.
                 for k, v in tool_result.usage.items():
@@ -860,14 +835,13 @@ class Agent(BaseStep):
             # Run the llm again.
             llm_i_path = f"{self.path}.llm-{i}"
 
-            llm_start_event = StartEvent(
+            llm_start_event = await StartEvent.build(
                 run_id=context.id,
                 path=llm_i_path,
                 status_text="Thinking...",
             )
-            llm_start_event_dump = dump(llm_start_event, context=context)
 
-            logger.info("start_event", start_event=llm_start_event_dump)
+            logger.info("start_event", start_event=llm_start_event.dump)
             yield llm_start_event
 
             async for llm_output in self._run_llm(
@@ -881,28 +855,16 @@ class Agent(BaseStep):
                 else:
                     llm_chunk = llm_output
 
-                    llm_chunk_event = ChunkEvent(
+                    llm_chunk_event = await ChunkEvent.build(
                         run_id=context.id,
                         path=llm_i_path,
                         chunk=llm_chunk.output,
                     )
-                    llm_chunk_event_dump = dump(llm_chunk_event, context=context)
 
-                    logger.info("chunk_event", chunk_event=llm_chunk_event_dump)
+                    logger.info("chunk_event", chunk_event=llm_chunk_event.dump)
                     yield llm_chunk_event
 
-                    # # If the LLM is returning a stream, that indicates it's not going to use a tool.
-                    # # We're safe streaming the chunks as the final agent response.
-                    # agent_chunk_event = ChunkEvent(
-                    #     run_id=context.id,
-                    #     path=self.path,
-                    #     chunk=llm_chunk.output,
-                    # )
-
-                    # logger.info("chunk_event", chunk_event=agent_chunk_event)
-                    # yield agent_chunk_event
-
-            llm_output_event = OutputEvent(
+            llm_output_event = await OutputEvent.build(
                 run_id=context.id,
                 path=llm_i_path,
                 input=llm_result.input,
@@ -912,13 +874,12 @@ class Agent(BaseStep):
                 t1=llm_result.t1,
                 usage=llm_result.usage,
             )
-            llm_output_event_dump = dump(llm_output_event, context=context)
 
-            logger.info("output_event", output_event=llm_output_event_dump)
+            logger.info("output_event", output_event=llm_output_event.dump)
             yield llm_output_event
 
             # Store the trace of the LLM step.
-            run_steps[llm_i_path] = llm_output_event_dump
+            run_steps[llm_i_path] = llm_output_event.dump
 
             # Aggregate the usage of the LLM step.
             for k, v in llm_result.usage.items():
@@ -947,10 +908,7 @@ class Agent(BaseStep):
                     
                     # We don't want to cancel the execution if this errors. 
                     try:
-                        if self._is_state_saver_put_async:
-                            await self.state_saver.put(snapshot=snapshot, context=context)
-                        else:
-                            self.state_saver.put(snapshot=snapshot, context=context)
+                        await self.state_saver.put(snapshot=snapshot)
                     except Exception as err:
                         logger.error("put_memory_error", err=err)
 
@@ -995,14 +953,12 @@ class Agent(BaseStep):
 
             # We don't want to cancel the execution if this errors. 
             try:
-                if self._is_state_saver_put_async:
-                    await self.state_saver.put(snapshot=snapshot, context=context)
-                else:
-                    self.state_saver.put(snapshot=snapshot, context=context)
+                await self.state_saver.put(snapshot=snapshot)
+                set_run_context(RunContext(parent_id=snapshot.id))
             except Exception as err:
                 logger.error("put_memory_error", err=err)
 
-        agent_output_event = OutputEvent(
+        agent_output_event = await OutputEvent.build(
             run_id=context.id,
             path=self.path,
             input=agent_input,
@@ -1012,20 +968,16 @@ class Agent(BaseStep):
             t1=t1,
             usage=run_usage,
         )
-        agent_output_event_dump = dump(agent_output_event, context=context)
 
-        logger.info("output_event", output_event=agent_output_event_dump)
+        logger.info("output_event", output_event=agent_output_event.dump)
         yield agent_output_event
 
 
-    async def _run_remote(
-        self,
-        context: RunContext | None = None,
-        **kwargs: Any,
-    ) -> Any: 
+    async def _run_remote(self, **kwargs: Any) -> Any: 
         """Runs the agent remotely. The agent must be deployed in the Timbal platform."""
         prompt = kwargs.pop("prompt", None)
         prompt = Message.validate(prompt)
+        prompt = await dump(prompt)
 
         path = f"orgs/{self.remote_config.org_id}/apps/{self.remote_config.app_id}/runs/stream"
         payload = {
@@ -1033,7 +985,7 @@ class Agent(BaseStep):
             # "parent_id": parent_id,
             # "group_id": group_id,
             "input": {
-                "prompt": dump(prompt, context=context),
+                "prompt": prompt,
                 **kwargs,
             },
         }
@@ -1102,20 +1054,19 @@ class Agent(BaseStep):
             - Supports continuation of previous conversations
             - Handles both streaming and non-streaming LLMs
         """
-        if context is None:
-            context = RunContext(id=uuid7(as_type="str"))
-        elif context.id is None:
-            context.id = uuid7(as_type="str")
-
-        # Set the run context for the duration of the agent run.
-        set_run_context(context)
+        run_context = get_run_context()
+        if not run_context:
+            run_context = RunContext(id=uuid7(as_type="str"))
+            set_run_context(run_context)
+        if not run_context.id:
+            run_context.id = uuid7(as_type="str")
 
         if self.remote_config:
-            async for event in self._run_remote(context=context, **kwargs):
+            async for event in self._run_remote(**kwargs):
                 event = TypeAdapter(Event).validate_python(event)
                 yield event
         else:
-            async for event in self._run_local(context=context, **kwargs):
+            async for event in self._run_local(**kwargs):
                 yield event
 
     
