@@ -278,3 +278,63 @@ Only mark as incorrect if the steps are missing key actions, are irrelevant, or
     return Validator(validator, "semantic", ref)
 
 
+def contains_any_output(ref: str | list[str]):
+    """Validate that the message contains any of the given substrings (OR logic)."""
+    if not isinstance(ref, list):
+        ref = [ref]
+    
+    ref = [str(v) for v in ref]
+
+    def validator(message: Message):
+        message_text = message.collect_text()
+        if not message_text:
+            raise EvalError("Message does not contain any text to validate.")
+
+        for v in ref:
+            if v in message_text:
+                return  # Found at least one, validation passes
+
+        # If we get here, none were found
+        ref_str = "', '".join(ref)
+        raise EvalError(f"Message does not contain any of: '{ref_str}'.")
+
+    return Validator(validator, "contains_any", ref)
+
+
+def contains_any_steps(ref: list[dict]):
+    """
+    Validate that the steps contain any of the given tool names and, optionally, input key-value substrings (OR logic).
+    Each ref item should be a dict with 'name' (tool name) and optionally 'input' (a dict of expected input substrings).
+    """
+    if not isinstance(ref, list):
+        raise ValueError(f"Invalid contains_any_steps validator: expected list, got {type(ref)}")
+
+    def validator(steps: list[dict]):
+        for expected in ref:
+            tool_name = expected.get("name")
+            input_dict = expected.get("input", {})
+            
+            for step in steps:
+                if step.get("tool") == tool_name:
+                    # If no input dict specified, just match tool
+                    if not input_dict:
+                        return  # Found a match, validation passes
+                    # Otherwise, check all input keys/values
+                    step_input = step.get("input", {})
+                    all_match = True
+                    for k, v in input_dict.items():
+                        step_val = step_input.get(k)
+                        if step_val is None or str(v) not in str(step_val):
+                            all_match = False
+                            break
+                    if all_match:
+                        return  # Found a match, validation passes
+        
+        # If we get here, none were found
+        tool_names = [expected.get("name") for expected in ref]
+        tools_str = "', '".join(tool_names)
+        raise EvalError(f"No step found with any of the tools: '{tools_str}'.")
+    
+    return Validator(validator, "contains_any_steps", ref)
+
+
