@@ -1,6 +1,5 @@
 from abc import ABC, abstractmethod
 
-from ..context import RunContext
 from ..snapshot import Snapshot
 
 
@@ -8,42 +7,26 @@ class BaseSaver(ABC):
     """Base class for creating a Flow state saver.
 
     State savers persist Flow states across multiple interactions.
-
-    Note:
-        When creating a custom state saver, consider implementing async
-        versions to avoid blocking the main thread.
     """
 
-
     @abstractmethod 
-    def get_last(
-        self, 
-        path: str,
-        context: RunContext,
-    ) -> Snapshot | None:
+    async def get_last(self, path: str) -> Snapshot | None:
         """Retrieve the last snapshot matching the specified criteria.
 
         Args:
             path: Flows are nested structures. We don't ensure id uniqueness across nested subflows.
                   This path ensures uniqueness of every different BaseStep in the flow.
-            context: The run context. For instance, if we're looking for a specific parent id or a group id.
 
         Returns:
             The last snapshot matching the specified criteria.
         """
         pass
 
-
     @abstractmethod
-    def put(
-        self, 
-        snapshot: Snapshot, 
-        context: RunContext,
-    ) -> None:
+    async def put(self, snapshot: Snapshot) -> None:
         """Store a new snapshot.
 
         Args:
             snapshot: The Snapshot object to store.
-            context: The run context. Might be used for authentication to an external service.
         """
         pass
