@@ -1,4 +1,6 @@
 # ruff: noqa: F401
+from typing import Annotated
+
 from anthropic.types import (
     RawContentBlockDeltaEvent,
     RawContentBlockStartEvent,
@@ -8,6 +10,11 @@ from anthropic.types import (
     RawMessageStopEvent,
 )
 from openai.types.chat import ChatCompletionChunk
+from pydantic import Field
+
+from .chunk import ChunkEvent
+from .output import OutputEvent
+from .start import StartEvent
 
 # Create a type alias for Anthropic events
 AnthropicEvent = (
@@ -19,11 +26,12 @@ AnthropicEvent = (
     RawMessageStopEvent
 )
 
-
 # Create a type alias for OpenAI events (in the future we may have more than one)
 OpenAIEvent = ChatCompletionChunk
 
-
-from .chunk import ChunkEvent
-from .output import OutputEvent
-from .start import StartEvent
+# Create a discriminated union of all possible event types.
+# Pydantic will use the 'type' field to determine which model to use.
+Event = Annotated[
+    StartEvent | OutputEvent | ChunkEvent,
+    Field(discriminator="type"),
+]
